# Copyright (c) 2014 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# TODO: add a -v --verbosity flag.
import logging
import os
from gettext import gettext as _

import gi
from gi.repository import Gio
from gi.repository import GObject

gi.require_version("GdkPixbuf", "2.0")
gi.require_version("Gtk", "3.0")
gi.require_version("Notify", "0.7")
gi.require_version("AppIndicator3", "0.1")

from gi.repository import AppIndicator3  # noqa: E402
from gi.repository import GdkPixbuf  # noqa: E402
from gi.repository import Gtk  # noqa: E402
from gi.repository.Notify import Notification  # noqa: E402
from gi.repository.Notify import init as notify_init  # noqa: E402

from caffeine import __version__  # noqa: E402
from caffeine.core import Caffeine  # noqa: E402
from caffeine.icons import generic_icon  # noqa: E402
from caffeine.icons import get_icon_pixbuf  # noqa: E402
from caffeine.paths import get_blacklist_file_audio  # noqa: E402
from caffeine.paths import get_glade_file  # noqa: E402
from caffeine.paths import get_whitelist_file  # noqa: E402
from caffeine.procmanager import ProcManager  # noqa: E402

logger = logging.getLogger(__name__)

icon_theme = Gtk.IconTheme.get_default()
try:
    generic = icon_theme.load_icon(
        "application-x-executable",
        16,
        Gtk.IconLookupFlags.NO_SVG,
    )
except GObject.GError:
    generic = GdkPixbuf.Pixbuf.new_from_file(generic_icon)

cached_icons = {"generic": generic}


# Set this environment variable to any value to use the legacy indicator.
use_legacy_indicator = os.environ.get("CAFFEINE_LEGACY_TRAY") is not None


# FIXME: this does not work for any of the cases I tried.
def get_icon_for_process(proc_name: str):

    global cached_icons
    global generic

    possible_icon_names = proc_name.split("-")
    possible_icon_names.insert(0, proc_name)

    for icon_name in possible_icon_names:

        icon_name = icon_name.split("/")[-1]

        # Check to see if we have loaded this already.
        try:
            return cached_icons[icon_name]
        except KeyError:
            pass
        try:
            icon = icon_theme.load_icon(proc_name, 16, Gtk.IconLookupFlags.NO_SVG)
            cached_icons[icon_name] = icon
            return icon

        except GObject.GError:
            cached_icons[icon_name] = generic

    return cached_icons["generic"]


class ProcAdd:
    def __init__(self):
        self.running_id = None

        self.user_set = True

        builder = Gtk.Builder()
        builder.add_from_file(get_glade_file("proc.glade"))

        get = builder.get_object

        self.dialog = get("dialog")
        self.entry = get("entry")
        self.entry.grab_focus()

        builder.connect_signals(self)

    def run(self):

        self.entry.set_text("")

        response = self.dialog.run()
        self.hide()
        return response

    def get_process_name(self):
        return self.entry.get_text().lower().strip()

    def on_add_button_clicked(self, button, data=None):
        self.dialog.hide()

    def on_cancel_button_clicked(self, button, data=None):
        self.dialog.hide()

    def hide(self):
        self.dialog.hide()

    def on_window_delete_event(self, window, data=None):
        window.hide_on_delete()
        # Returning True stops the window from being destroyed.
        return True


class GUI:
    def __init__(self, show_preferences=False, **kwargs):
        # object to manage processes to activate for.
        self.__process_manager = ProcManager(persistence_file=get_whitelist_file())
        self.__process_manager_audio = ProcManager(
            persistence_file=get_blacklist_file_audio()
        )

        self.__core = Caffeine(
            process_manager=self.__process_manager,
            process_manager_audio=self.__process_manager_audio,
            **kwargs
        )

        self.__core.connect("activation-toggled", self.on_activation_toggled)
        self.ProcAdd = ProcAdd()

        # XXX: Do we want to change this to caffeine-ng?
        settings = Gio.Settings.new("net.launchpad.caffeine")

        builder = Gtk.Builder()
        builder.add_from_file(get_glade_file("GUI.glade"))

        about = builder.get_object("aboutdialog")
        about.set_version(__version__)

        show_tray_icon = settings.get_boolean("show-tray-icon")
        show_notification = settings.get_boolean("show-notification")
        audio_peak_filtering_active = settings.get_boolean("audio-peak-filtering")
        self.__core.set_audio_peak_filtering_active(audio_peak_filtering_active)

        if not use_legacy_indicator:
            self.AppInd = AppIndicator3.Indicator.new(
                "caffeine",
                "caffeine-cup-empty",
                AppIndicator3.IndicatorCategory.APPLICATION_STATUS,
            )

            self.AppInd.set_status(
                [
                    AppIndicator3.IndicatorStatus.PASSIVE,
                    AppIndicator3.IndicatorStatus.ACTIVE,
                ][show_tray_icon]
            )
        else:
            # IMPORTANT:
            # status icon must be a instance variable  (ie self.)or else it
            # gets thrown out with the garbage, and won't be seen.

            self.status_icon = builder.get_object("statusicon")
            self.status_icon.set_visible(show_tray_icon)

        if (
            show_tray_icon is False
            and show_notification is True
            and show_preferences is False
        ):
            notify_init("caffeine-ng")
            note = Notification.new(
                _("Caffeine is running"),
                _("To show the tray icon, \nrun ")
                + "'caffeine -p' "
                + _("or open Caffeine Preferences from your system menu."),
                "caffeine",
            )

            note.show()

        self.activate_menuitem = builder.get_object("activate_menuitem")

        self.set_icon_is_activated(self.__core.get_activated())

        tooltip = self.__core.status_string
        if not tooltip:
            tooltip = _("Caffeine is dormant; powersaving is enabled")
        # self.status_icon.set_tooltip(tooltip)

        # popup menu
        self.menu = builder.get_object("popup_menu")
        self.menu.show()

        if not use_legacy_indicator:

            self.AppInd.set_menu(self.menu)

        #
        # configuration window widgets
        #
        proc_treeview = builder.get_object("treeview")
        self.selection = proc_treeview.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        proc_treeview_audio = builder.get_object("treeview_audio")
        self.selection_audio = proc_treeview_audio.get_selection()
        self.selection_audio.set_mode(Gtk.SelectionMode.MULTIPLE)

        self.proc_liststore = builder.get_object("proc_liststore")
        for name in self.__process_manager.get_process_list():
            self.proc_liststore.append([get_icon_for_process(name), name])
        self.proc_liststore_audio = builder.get_object("proc_liststore_audio")
        for name in self.__process_manager_audio.get_process_list():
            self.proc_liststore_audio.append([get_icon_for_process(name), name])

        # time_menuitem = builder.get_object("time_menuitem")

        # time_menuitem.set_submenu(submenu)

        # Preferences editor.
        self.window = builder.get_object("window")

        # set the icons for the window border.
        self.window.set_default_icon_list(
            [
                get_icon_pixbuf(16),
                get_icon_pixbuf(24),
                get_icon_pixbuf(32),
                get_icon_pixbuf(48),
            ]
        )

        self.trayicon_cb = builder.get_object("trayicon_cbutton")
        self.notification_cb = builder.get_object("notification_cbutton")
        self.audio_peak_filtering_cb = builder.get_object(
            "audio_peak_filtering_cbutton"
        )

        self.notification_cb.set_sensitive(not show_tray_icon)

        settings.connect("changed::show-tray-icon", self.on_trayicon_changed)
        settings.connect("changed::show-notification", self.on_notification_changed)
        settings.connect(
            "changed::audio-peak-filtering", self.on_audio_peak_filtering_changed
        )

        settings.bind(
            "show-tray-icon", self.trayicon_cb, "active", Gio.SettingsBindFlags.DEFAULT
        )
        settings.bind(
            "show-notification",
            self.notification_cb,
            "active",
            Gio.SettingsBindFlags.DEFAULT,
        )
        settings.bind(
            "audio-peak-filtering",
            self.audio_peak_filtering_cb,
            "active",
            Gio.SettingsBindFlags.DEFAULT,
        )

        # about dialog
        self.about_dialog = builder.get_object("aboutdialog")
        self.about_dialog.set_translator_credits(_("translator-credits"))

        # other time selector
        self.othertime_dialog = builder.get_object("othertime_dialog")
        self.othertime_hours = builder.get_object("hours_spin")
        self.othertime_minutes = builder.get_object("minutes_spin")

        if use_legacy_indicator:
            # Handle mouse clicks on status_icon
            # left click
            self.status_icon.connect("activate", self.on_L_click)
            # right click
            self.status_icon.connect("popup-menu", self.on_R_click)

        builder.connect_signals(self)

    def setActive(self, active: bool):
        self.__core.set_activated(active)

    def timed_activation(self, time):
        self.__core.timed_activation(time)

    def toggle_activated(self):
        """Toggles whether screen saver prevention is active."""
        self.__core.toggle_activated()

    def on_activation_toggled(self, source, active, tooltip):
        self.set_icon_is_activated(active)
        # self.status_icon.set_tooltip(tooltip)

    def set_icon_is_activated(self, activated):
        # toggle the icon, indexing with a bool.
        icon_name = ["caffeine-cup-empty", "caffeine-cup-full"][activated]

        if not use_legacy_indicator:
            self.AppInd.set_icon(icon_name)
        else:
            self.status_icon.set_from_icon_name(icon_name)

        label = [_("Enable Caffeine"), _("Disable Caffeine")]
        self.activate_menuitem.set_label(label[self.__core.get_activated()])

    # Callbacks
    def on_L_click(self, status_icon, data=None):
        logger.info("User has clicked the Caffeine icon")
        self.toggle_activated()

    def on_R_click(self, status_icon, mbutton, time, data=None):
        # popdown menu
        self.menu.show_all()

        self.menu.popup(None, None, None, self.status_icon, 3, time)

    # Window callbacks
    def on_add_button_clicked(self, button, data=None):
        response = self.ProcAdd.run()
        if response == 1:
            proc_name = self.ProcAdd.get_process_name()
            if proc_name:
                self.proc_liststore.append([get_icon_for_process(proc_name), proc_name])

                self.__process_manager.add_proc(proc_name)

    def on_remove_button_clicked(self, button, data=None):
        model, paths = self.selection.get_selected_rows()
        paths.reverse()
        for path in paths:
            self.__process_manager.remove_proc(model[path][1])
            model.remove(model.get_iter(path))

    def on_add_button_audio_clicked(self, button, data=None):
        response = self.ProcAdd.run()
        if response == 1:
            proc_name = self.ProcAdd.get_process_name()
            if proc_name:
                self.proc_liststore_audio.append(
                    [get_icon_for_process(proc_name), proc_name]
                )

                self.__process_manager_audio.add_proc(proc_name)

    def on_remove_button_audio_clicked(self, button, data=None):
        model, paths = self.selection_audio.get_selected_rows()
        paths.reverse()
        for path in paths:
            self.__process_manager_audio.remove_proc(model[path][1])
            model.remove(model.get_iter(path))

    def on_window_delete_event(self, window, data=None):
        window.hide_on_delete()
        # Returning True stops the window from being destroyed.
        return True

    def on_close_button_clicked(self, button, data=None):
        self.window.hide()

    # Tray icon
    def on_trayicon_changed(self, settings, key, data=None):
        show_tray_icon = settings.get_boolean(key)

        if not use_legacy_indicator:
            self.AppInd.set_status(
                [
                    AppIndicator3.IndicatorStatus.PASSIVE,
                    AppIndicator3.IndicatorStatus.ACTIVE,
                ][show_tray_icon]
            )

        else:
            self.status_icon.set_visible(show_tray_icon)

        self.trayicon_cb.set_active(show_tray_icon)
        self.notification_cb.set_sensitive(not show_tray_icon)

    # Startup Notifications
    def on_notification_changed(self, settings, key, data=None):
        pass

    def on_audio_peak_filtering_changed(self, settings, key, data=None):
        audio_filtering_active = settings.get_boolean(key)
        self.audio_peak_filtering_cb.set_active(audio_filtering_active)
        self.__core.set_audio_peak_filtering_active(audio_filtering_active)

    # Menu callbacks
    def on_activate_menuitem_activate(self, menuitem, data=None):
        self.toggle_activated()

        label = [_("Enable Caffeine"), _("Disable Caffeine")]
        menuitem.set_label(label[self.__core.get_activated()])

    def on_time_menuitem_activate(self, menuitem, data=None):
        self.othertime_dialog.show_all()

    def on_prefs_menuitem_activate(self, menuitem, data=None):
        self.window.show_all()

    def on_about_menuitem_activate(self, menuitem, data=None):
        self.about_dialog.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
        self.about_dialog.run()
        self.about_dialog.hide()

    def on_othertime_delete_event(self, window, data=None):
        window.hide_on_delete()
        # Returning True stops the window from being destroyed.
        return True

    def on_othertime_cancel(self, widget, data=None):
        self.othertime_dialog.hide()

    def on_othertime_ok(self, widget, data=None):
        hours = int(self.othertime_hours.get_value())
        minutes = int(self.othertime_minutes.get_value())
        self.othertime_dialog.hide()
        time = hours * 60 * 60 + minutes * 60
        if time > 0:
            self.__core.timed_activation(time)

    def on_quit_menuitem_activate(self, menuitem, data=None):
        self.quit()

    def quit(self):
        # Do anything that needs to be done before quitting.
        logger.info("Caffeine is preparing to quit")

        # Make sure PM and SV is uninhibited
        self.__core.set_activated(False)
        self.__core.quit()
        Gtk.main_quit()

    def run(self):
        Gtk.main()



# Copyright (c) 2014 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import gettext
import locale

from caffeine.paths import LOCALE_PATH
from caffeine.version import version

__version__ = version


def init_translations() -> None:
    """Initialise translations. Should be called just once at startup."""

    GETTEXT_DOMAIN = "caffeine"
    locale.setlocale(locale.LC_ALL, "")

    for module in locale, gettext:
        module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
        module.textdomain(GETTEXT_DOMAIN)


init_translations()



# Copyright (c) 2014 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from os.path import join

from gi.repository import Gtk

from caffeine.paths import ICON_PATH
from caffeine.paths import IMAGE_PATH


def get_icon_pixbuf(size: int):
    """
    Returns the icon as a pixbuf.

    Returns caffeine's icon as a pixbuf. We wrap around GTK here, so the
    default theme's icon will be used, if any, or caffeine's own as a fallback.
    """
    icon_name = "caffeine"
    icon_theme = Gtk.IconTheme.get_default()

    icon_info = icon_theme.lookup_icon(icon_name, size, Gtk.IconLookupFlags.NO_SVG)

    if icon_info:
        # icon is found
        base_size = icon_info.get_base_size()
        if base_size != size:
            # No size x size icon in the users theme so use the default
            icon_theme = Gtk.IconTheme()
            icon_theme.set_search_path((ICON_PATH,))
    else:
        icon_theme.append_search_path(ICON_PATH)
        icon_info = icon_theme.lookup_icon(icon_name, size, Gtk.IconLookupFlags.NO_SVG)

    pixbuf = icon_theme.load_icon(icon_name, size, Gtk.IconLookupFlags.NO_SVG)

    return pixbuf


generic_icon = join(IMAGE_PATH, "application-x-executable.png")

full_cup_icon = join(IMAGE_PATH, "Full_Cup.svg")
empty_cup_icon = join(IMAGE_PATH, "Empty_Cup.svg")



"""Triggers are different events or states that auto-activate caffeine."""
import logging
import os
from abc import ABC
from abc import abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Callable

from ewmh import EWMH
from pulsectl import Pulse
from pulsectl.pulsectl import PulseIndexError

from caffeine import utils
from caffeine.procmanager import ProcManager  # noqa: E402

logger = logging.getLogger(__name__)


class DesiredState(Enum):
    UNINHIBITED = 0  # Don't inhibit anything.
    INHIBIT_SLEEP = 5  # Only inhibit sleeping (screen saver can go off).
    INHIBIT_ALL = 10  # Inhibit both screen saver and sleeping.

    def __gt__(self, other):
        if self.__class__ is other.__class__:
            return self.value > other.value
        return NotImplemented


class Trigger(ABC):
    """Triggers are "sources" that indicate that inhibition is desireable."""

    @abstractmethod
    def run(self) -> DesiredState:
        """Return the desired state right now.

        This method will be called periodically, and the trigger should return the
        desired state at the time of the call.
        """


class ManualTrigger(Trigger):
    active = False

    def run(self) -> DesiredState:
        if self.active:
            return DesiredState.INHIBIT_ALL
        else:
            return DesiredState.UNINHIBITED


@dataclass
class WhiteListTrigger(Trigger):
    process_manager: ProcManager

    def run(self) -> DesiredState:
        """Determine if one of the whitelisted processes is running."""

        for proc in self.process_manager.get_process_list():
            if utils.is_process_running(proc):
                logger.info("Process %s detected. Inhibiting.")
                return DesiredState.INHIBIT_ALL

        return DesiredState.UNINHIBITED


class FullscreenTrigger(Trigger):
    def __init__(self):
        if os.environ.get("WAYLAND_DISPLAY") is None:
            self._ewmh = EWMH()
        else:
            logger.info("Running on Wayland; fullscreen trigger won't work.")
            self._ewmh = None

    def run(self) -> DesiredState:
        """Determine if a fullscreen application is running."""
        inhibit = False

        if self._ewmh:
            window = self._ewmh.getActiveWindow()

            # ewmh.getWmState(window) returns None is scenarios where
            # ewmh.getWmState(window, str=True) throws an exception
            # (it's a bug in pyewmh):
            if window and self._ewmh.getWmState(window):
                wm_state = self._ewmh.getWmState(window, True)
                inhibit = "_NET_WM_STATE_FULLSCREEN" in wm_state

        if inhibit:
            logger.info("Fullscreen window detected.")
            return DesiredState.INHIBIT_ALL
        else:
            return DesiredState.UNINHIBITED


class PulseAudioTrigger(Trigger):
    def __init__(
        self,
        process_manager: ProcManager,
        audio_peak_filtering_active_getter: Callable[[], bool],
    ) -> None:
        self.__process_manager = process_manager
        self.__audio_peak_filtering_active_getter = audio_peak_filtering_active_getter

    @property
    def __audio_peak_filtering_active(self) -> bool:
        return self.__audio_peak_filtering_active_getter()

    def run(self) -> DesiredState:
        # Let's look for playing audio:
        # Number of supposed audio only streams.  We can turn the screen off
        # for those:
        music_procs = 0
        # Number of all audio streams including videos. We keep the screen on
        # here:
        screen_relevant_procs = 0
        # Applications currently playing audio.
        active_applications = []
        # Applications whose audio activity is ignored
        ignored_applications = self.__process_manager.get_process_list()

        # Get all audio playback streams
        # Music players seem to use the music role. We can turn the screen
        # off there. Keep the screen on for audio without music role,
        # as they might be videos
        with Pulse() as pulseaudio:
            for application_output in pulseaudio.sink_input_list():
                if (
                    not application_output.mute  # application audio is not muted
                    and not application_output.corked  # application audio is not paused
                    and not pulseaudio.sink_info(
                        application_output.sink
                    ).mute  # system audio is not muted
                ):
                    application_name = application_output.proplist.get(
                        "application.process.binary", "no name"
                    )
                    if application_name in ignored_applications:
                        continue
                    if self.__audio_peak_filtering_active:
                        # ignore silent sinks
                        sink_source = pulseaudio.sink_info(
                            application_output.sink
                        ).monitor_source
                        sink_peak = pulseaudio.get_peak_sample(sink_source, 0.4)
                        if not sink_peak > 0:
                            continue
                    if application_output.proplist.get("media.role") == "music":
                        # seems to be audio only
                        music_procs += 1
                    else:
                        # Video or other audio source
                        screen_relevant_procs += 1
                    # Save the application's process name
                    active_applications.append(application_name)

            # Get all audio recording streams
            for application_input in pulseaudio.source_output_list():
                try:
                    system_input_muted = pulseaudio.source_info(
                        application_input.source
                    ).mute
                except PulseIndexError:
                    system_input_muted = False
                if (
                    not application_input.mute  # application input is not muted
                    and not system_input_muted
                ):
                    application_name = application_input.proplist.get(
                        "application.process.binary", "no name"
                    )
                    if application_name in ignored_applications:
                        continue
                    if self.__audio_peak_filtering_active:
                        # ignore silent sources
                        source_peak = pulseaudio.get_peak_sample(
                            application_input.source, 0.1
                        )
                        if not (source_peak > 0):
                            continue
                    # Treat recordings as video because likely you don't
                    # want to turn the screen of while recording
                    screen_relevant_procs += 1
                    # Save the application's process name
                    active_applications.append(application_name)

        if screen_relevant_procs > 0:
            logger.debug(f"Video playback detected ({', '.join(active_applications)}).")
            return DesiredState.INHIBIT_ALL
        elif music_procs > 0:
            logger.debug(f"Audio playback detected ({', '.join(active_applications)}).")
            return DesiredState.INHIBIT_SLEEP
        else:
            return DesiredState.UNINHIBITED



import ctypes
import logging
import signal

import click
from setproctitle import setproctitle

from caffeine.applicationinstance import ApplicationInstance
from caffeine.main import GUI

logger = logging.getLogger(__name__)


@click.group(invoke_without_command=True)
@click.version_option(prog_name="caffeine")
@click.option("--verbose", "-v", is_flag=True)
@click.pass_context
def cli(ctx, verbose):
    setproctitle("caffeine-ng")
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # register the process id as 'caffeine'
    libc = ctypes.cdll.LoadLibrary("libc.so.6")
    libc.prctl(15, "caffeine", 0, 0, 0)

    if verbose:
        logging.basicConfig(level=logging.DEBUG)
        logger.debug("Running with --verbose.")

    ctx.obj = ApplicationInstance("caffeine-ng")

    if not ctx.invoked_subcommand:
        ctx.invoke(cli.commands["start"])


@cli.command()
@click.option(
    "--activate",
    "-a",
    is_flag=True,
    help="Immediately disable power management and screen saving.",
)
@click.option(
    "--deactivate",
    "-d",
    is_flag=True,
    help="Immediately re-enable power management and screen saving.",
)
@click.option(
    "--kill",
    "-k",
    is_flag=True,
    help="Kill any running instance of caffeine.",
)
@click.option(
    "--time",
    "-t",
    # XXX: There's a param issing to make this take values.
    help="Use with -a. Activate caffeine for HH:MM.",
    metavar="HH:MM",
)
@click.option(
    "--preferences",
    "-p",
    is_flag=True,
    help="Start with the Preferences dialog open.",
)
@click.option(
    "--pulseaudio/--no-pulseaudio",
    default=True,
    help=(
        "Inhibit when pulseaudio is in use. "
        "Only the screensaver (i.e.: not suspension) is inhibited when audio is playing."
    ),
)
@click.option(
    "--whitelist/--no-whitelist",
    default=True,
    help="Inhibit based on whitelisted applications.",
)
@click.option(
    "--fullscreen/--no-fullscreen",
    default=True,
    help="Inhibit when a fullscreen application is detected.",
)
@click.pass_obj
def start(
    app: ApplicationInstance,
    activate: bool,
    deactivate: bool,
    kill: bool,
    time: str,
    preferences: bool,
    pulseaudio: bool,
    whitelist: bool,
    fullscreen: bool,
):
    """Start caffeine."""
    if kill:
        app.kill()
    elif app.is_running():
        raise click.ClickException("Caffeine is already running.")

    main = GUI(
        show_preferences=preferences,
        pulseaudio=pulseaudio,
        whitelist=whitelist,
        fullscreen=fullscreen,
    )
    if activate:
        main.setActive(True)

    if activate and time:
        parts = time.split(":")
        if len(parts) != 2:
            raise click.ClickException("-t argument must be in the hour:minute format.")

        try:
            hours = int(parts[0])
            minutes = int(parts[1])
        except ValueError:
            raise click.ClickException("Invalid time argument.")

        main.timed_activation((hours * 3600.0) + (minutes * 60.0))

    if preferences:
        main.window.show_all()

    with app.pid_file():
        main.run()


@cli.command()
@click.pass_obj
def kill(app: ApplicationInstance):
    """Kill any running instances of caffeine and exit."""
    if app.is_running():
        app.kill()
    else:
        raise click.ClickException("Caffeine is not running.")


if __name__ == "__main__":
    cli()



# Copyright (c) 2014 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import logging
import os
from typing import Generator

logger = logging.getLogger(__name__)


def get_process_name(pid: int) -> str:
    """Gets process name from process id."""

    truncated_process_name = open("/proc/%s/status" % pid).readline()[6:-1]
    process_name = truncated_process_name

    line = open("/proc/%s/cmdline" % pid).readline()
    parts = line.split("\x00")
    for part in parts:
        cmd_name = part.split("/")[-1]

        if cmd_name.startswith(truncated_process_name):
            process_name = cmd_name

    return process_name


def get_processes() -> Generator[str, None, None]:
    """Return a generator of names of running processes."""

    for pid in os.listdir("/proc/"):
        try:
            num_pid = int(pid)
        except ValueError:
            continue

        try:
            process_name = get_process_name(num_pid).lower()
        except Exception:
            logger.exception(f"Failed to get name for process {num_pid}")
            continue

        yield process_name


def is_process_running(name: str) -> bool:
    """Return True is a process with the given name is running."""

    return name in get_processes()



# Copyright (c) 2014-2015 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import logging
import os.path
from gettext import gettext as _
from threading import Timer
from typing import List
from typing import Optional

from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Notify

from caffeine.icons import empty_cup_icon
from caffeine.icons import full_cup_icon
from caffeine.inhibitors import DpmsInhibitor
from caffeine.inhibitors import GnomeInhibitor
from caffeine.inhibitors import XautolockInhibitor
from caffeine.inhibitors import XdgPowerManagmentInhibitor
from caffeine.inhibitors import XdgScreenSaverInhibitor
from caffeine.inhibitors import XfceInhibitor
from caffeine.inhibitors import XidlehookInhibitor
from caffeine.inhibitors import XorgInhibitor
from caffeine.inhibitors import XssInhibitor
from caffeine.procmanager import ProcManager
from caffeine.triggers import DesiredState
from caffeine.triggers import FullscreenTrigger
from caffeine.triggers import ManualTrigger
from caffeine.triggers import PulseAudioTrigger
from caffeine.triggers import Trigger
from caffeine.triggers import WhiteListTrigger

os.chdir(os.path.abspath(os.path.dirname(__file__)))

logger = logging.getLogger(__name__)


class Caffeine(GObject.GObject):
    """Implements the main caffeine service itself."""

    timer: Optional[Timer]

    def __init__(
        self,
        process_manager: ProcManager,
        process_manager_audio: ProcManager,
        pulseaudio: bool,
        whitelist: bool,
        fullscreen: bool,
    ):
        """Main caffeine worker.

        :param pulseaudio: Whether pulseaudio support should be enabled.
        :param whitelist: Whether whitelist support should be enabled.
        """
        GObject.GObject.__init__(self)

        self.__inhibitors = [
            GnomeInhibitor(),
            XdgPowerManagmentInhibitor(),
            XssInhibitor(),
            XorgInhibitor(),
            XautolockInhibitor(),
            XfceInhibitor(),
            XidlehookInhibitor(),
            XdgScreenSaverInhibitor(),
            DpmsInhibitor(),
        ]

        self.__process_manager = process_manager
        self.__process_manager_audio = process_manager_audio

        self.__audio_peak_filtering_active = True

        self._manual_trigger = ManualTrigger()
        self.triggers: List[Trigger] = [self._manual_trigger]
        if whitelist:
            self.triggers.append(WhiteListTrigger(self.__process_manager))
        if fullscreen:
            self.triggers.append(FullscreenTrigger())
        if pulseaudio:
            self.triggers.append(
                PulseAudioTrigger(
                    process_manager=self.__process_manager_audio,
                    audio_peak_filtering_active_getter=lambda: self.__audio_peak_filtering_active,
                )
            )

        logger.info("Running with triggers: %r.", self.triggers)

        # The initial state is uninhibited.
        self.desired_state = DesiredState.UNINHIBITED

        # Status string (XXX: Let's double check how well this is working).
        self.status_string = "Caffeine is starting up..."

        # Number of procs playing audio but nothing visual. This is a special
        # case where we want the screen to turn off while still preventing
        # the computer from suspending
        self.music_procs = 0

        self.timer = None
        self.notification = None

        # FIXME: add capability to xdg-screensaver to report timeout.
        GLib.timeout_add(10000, self.run_all_triggers)

        logger.info(self.status_string)

    def run_all_triggers(self, show_notification=False) -> None:
        """Runs all triggers to determine the currently desired status."""
        inhibit = DesiredState.UNINHIBITED

        for trigger in self.triggers:
            inhibit = max(inhibit, trigger.run())

            if inhibit == DesiredState.INHIBIT_ALL:
                logger.debug("%s requested %s.", trigger, inhibit)
                break

        logger.info(f"Desired state is: {inhibit}")
        self.desired_state = inhibit
        self.apply_desired_status(show_notification)

    def quit(self) -> None:
        """
        Cancels any timer thread running so the program can quit right away.
        """
        if self.timer:
            self.timer.cancel()

    def _notify(self, message: str, icon: str, title: str = "Caffeine") -> None:
        """Easy way to use pynotify."""

        Notify.init("Caffeine")
        if self.notification:
            self.notification.update(title, message, icon)
        else:
            self.notification = Notify.Notification.new(title, message, icon)

        if self.notification:
            self.notification.show()

    def timed_activation(self, time: int, show_notification=True) -> None:
        """Toggle inhibition after a given amount of seconds."""
        message = (
            _("Timed activation set; ")
            + _("Caffeine will prevent powersaving for the next ")
            + str(time)
        )

        logger.info("Timed activation set for " + str(time))

        if self.status_string == "":
            self.status_string = _("Activated for ") + str(time)

        self.set_activated(True)
        self.run_all_triggers()

        if show_notification:
            self._notify(message, full_cup_icon)

        # and deactivate after time has passed.
        # Stop already running timer
        if self.timer:
            interval = self.timer.interval  # type: ignore
            logger.info(
                "Previous timed activation cancelled due to a "
                "second timed activation request "
                f"(was set for {interval} or {time} seconds )"
            )
            self.timer.cancel()

        self.timer = Timer(time, self._deactivate, args=[show_notification])
        self.timer.name = "Active"
        self.timer.start()

    def _deactivate(self, show_notification: bool) -> None:
        """Called when the timer finished running."""

        self._manual_trigger.active = False
        interval = self.timer.interval  # type: ignore
        message = str(interval) + _(" have elapsed; powersaving is re-enabled")

        logger.info(
            "Timed activation period ("
            + str(self.timer.interval)  # type: ignore
            + ") has elapsed"
        )

        if show_notification:
            self._notify(message, empty_cup_icon)

        self.timer = None
        self.run_all_triggers()

    def set_activated(self, activated: bool) -> None:
        """Set manual activation to the provided value."""

        if not activated and self.timer:
            # If manually deactivating, cancel timers.
            self.cancel_timer()

        # Update actual status:
        self._manual_trigger.active = activated

        # Emit signal so the UI updates.
        self.emit(
            "activation-toggled",
            self.desired_state != DesiredState.UNINHIBITED,
            self.status_string,
        )

    def get_activated(self) -> bool:
        """Returns True if inhibition was manually activated."""
        return self._manual_trigger.active

    def toggle_activated(self, show_notification=True):
        """Toggle manual inhibition."""

        self.set_activated(not self.get_activated())
        self.run_all_triggers(show_notification)

    def cancel_timer(self, note=True):
        """Cancel a running timer.

        This cancellation is due to user interaction, generally, toggling a
        timed activation.

        :param note: Whether a notification should be shown.
        """

        # If the user manually disables caffeine, we should also
        # cancel the timer for timed activation.

        if self.timer is not None:
            interval: int = self.timer.interval  # type: ignore
            message = _("Timed activation cancelled (was set for ") + f"{interval})"

            logger.info("Timed cancelled (was set for %d).", interval)

            if note:
                self._notify(message, empty_cup_icon)

            self.timer.cancel()
            self.timer = None

        # Re run all triggers...
        self.run_all_triggers()

    def apply_desired_status(self, show_notification=False) -> None:
        """Applies the currently desired status."""

        inhibit_sleep = self.desired_state in (
            DesiredState.INHIBIT_SLEEP,
            DesiredState.INHIBIT_ALL,
        )
        inhibit_screen = self.desired_state == DesiredState.INHIBIT_ALL

        for inhibitor in self.__inhibitors:
            if inhibitor.applicable:
                if inhibitor.is_screen_inhibitor:
                    inhibitor.set(inhibit_screen)
                else:
                    inhibitor.set(inhibit_sleep)

                logger.info(f"{inhibitor} is applicable, state: {inhibitor.running}")

        if self.desired_state != DesiredState.UNINHIBITED:
            self.status_string = _("Caffeine is dormant; powersaving is enabled.")
        if self.desired_state != DesiredState.INHIBIT_SLEEP:
            self.status_string = _("Caffeine is preventing sleep only.")
        else:
            self.status_string = _("Caffeine is preventing all powersaving.")

        # Emit signal so the UI updates.
        self.emit(
            "activation-toggled",
            self.desired_state != DesiredState.UNINHIBITED,
            self.status_string,
        )

    def set_audio_peak_filtering_active(self, active: bool):
        self.__audio_peak_filtering_active = active


# register a signal
GObject.signal_new(
    "activation-toggled", Caffeine, GObject.SignalFlags.RUN_FIRST, None, [bool, str]
)



#!/usr/bin/env python
import os
import subprocess
import sys

if len(sys.argv) < 3:
    print("usage: ./compile_translations <program-name> <po directory>")
    sys.exit(1)

po_dir = sys.argv[-1]
prg_name = sys.argv[-2]


po_files = []
for dirpath, _dirnames, filenames in os.walk(po_dir):
    for file in filenames:
        if file.split(".")[-1] == "po":
            po_files.append(os.path.join(dirpath, file))

for po in po_files:
    lang = po.split("/")[-1]
    print("Compiling for Locale: " + "".join(lang.split(".")[:-1]))
    lang = lang.split("-")[-1]
    lang = lang.split(".")[0]
    lang = lang.strip()
    if not lang:
        continue

    lang_dir = os.path.join("share/locale", lang)

    if not os.path.isdir(lang_dir):
        os.mkdir(lang_dir)

    lang_lc_dir = os.path.join(lang_dir, "LC_MESSAGES")
    if not os.path.isdir(lang_lc_dir):
        os.mkdir(lang_lc_dir)

    cmd = ["msgfmt", po, "-o", os.path.join(lang_lc_dir, "caffeine.mo")]

    output = subprocess.check_output(cmd)



# Copyright (c) 2014-2021 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import fcntl
import logging
import os
from contextlib import contextmanager
from typing import Optional

try:
    from xdg.BaseDirectory import get_runtime_dir
except ModuleNotFoundError:
    from xdg import xdg_runtime_dir as get_runtime_dir

logger = logging.getLogger(__name__)


class AlreadyRunning(Exception):
    """Raised when another instance is already running."""

    def __str__(self):
        return "An instance is already running."


class ApplicationInstance:
    """Class used to handle one application instance mechanism."""

    def __init__(self, name: str):
        self.name = name
        self.pid_path = os.path.join(get_runtime_dir(), name, "pid")

    @property
    def pid(self) -> Optional[int]:
        """Return the PID of an already-running instance, if any."""
        try:
            with open(self.pid_path) as f:
                return int(f.read())
        except (ValueError, FileNotFoundError):
            return None

    def is_running(self) -> bool:
        """Return true if an instance is already running."""
        if self.pid:
            try:
                os.kill(self.pid, 0)
                logger.debug("An instance is already running: %d.", self.pid)
                return True
            except OSError:
                pass

        return False

    def kill(self) -> None:
        """Kill the currently running instance, if any."""
        if self.is_running():
            os.kill(self.pid, 9)  # type: ignore

    @contextmanager
    def pid_file(self):
        """Context manager to run code keeping a PID file alive."""

        pid_dir = os.path.dirname(self.pid_path)
        os.makedirs(pid_dir, exist_ok=True)

        handle = open(self.pid_path, "w+")

        try:
            fcntl.flock(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except BlockingIOError:
            raise AlreadyRunning()

        handle.seek(0)
        handle.write(str(os.getpid()))
        handle.flush()

        try:
            yield
        finally:
            try:
                os.remove(self.pid_path)
            except FileNotFoundError:
                pass



#!/usr/bin/env python
import os
from os import walk

from setuptools import find_packages
from setuptools import setup


def get_data_files():
    data_files = []

    for path, _dirs, files in walk("share"):
        target_path = os.path.join("/usr", path)

        if files:
            data_files.append((target_path, [os.path.join(path, f) for f in files]))

    # Install all icons for the package into /usr/share as well.
    # This is because the .desktop file actually uses them too.
    for path, _dirs, files in walk("caffeine/assets/icons"):
        target_path = os.path.join("/usr/share", path[16:])

        if files:
            data_files.append((target_path, [os.path.join(path, f) for f in files]))

    data_files.append(("/etc/xdg/autostart", ["share/applications/caffeine.desktop"]))

    return data_files


if __name__ == "__main__":
    setup(
        name="caffeine-ng",
        use_scm_version={
            "version_scheme": "post-release",
            "write_to": "caffeine/version.py",
        },
        description=("Tray bar app to temporarily inhibit screensaver and sleep mode."),
        long_description=open("README.rst").read(),
        author="The Caffeine Developers",
        author_email="hugo@barrera.io",
        maintainer="Hugo Osvaldo Barrera",
        maintainer_email="hugo@barrera.io",
        url="https://codeberg.org/WhyNotHugo/caffeine-ng",
        packages=find_packages(),
        include_package_data=True,
        data_files=get_data_files(),
        install_requires=[
            "click>=7.1,<8.0",
            "ewmh>=0.1.4",
            "pyxdg>=0.25",
            "setproctitle>=1.1.10",
            "wheel>=0.29.0",
            "pulsectl",
            "pygobject>=3.1.1,<4.0",
        ],
        # `console_scripts` is the same as `gui_scripts`.
        entry_points={"gui_scripts": ["caffeine = caffeine.cli:cli"]},
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Environment :: X11 Applications",
            "Environment :: X11 Applications :: GTK",
            "Intended Audience :: End Users/Desktop",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Operating System :: POSIX",
            "Operating System :: POSIX :: BSD",
            "Operating System :: POSIX :: Linux",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
        setup_requires=["setuptools_scm"],
    )



# Copyright (c) 2015-2022 Hugo Osvaldo Barrera <hugo@barrera.io>
#
# SPDX-License-Identifier: GPL-3.0+

import logging
import os
import threading
import time
from abc import ABC
from abc import abstractmethod

import dbus

logger = logging.getLogger(__name__)

INHIBITION_REASON = "Inhibited via libcaffeine"


class BaseInhibitor(ABC):
    running: bool

    def __init__(self):
        self.running = False

    def set(self, state: bool) -> None:
        if state:
            if not self.running:
                self.inhibit()
        else:
            if self.running:
                self.uninhibit()

    def is_screen_inhibitor(self):
        """Return True if this instance is a screen saver inhibitor.

        Inhibitor which are sleep inhibitors should return False.
        """
        return False

    @property
    @abstractmethod
    def applicable(self) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def inhibit(self, reason=INHIBITION_REASON) -> None:
        raise NotImplementedError()

    @abstractmethod
    def uninhibit(self) -> None:
        raise NotImplementedError()

    def __str__(self):
        return self.__class__.__name__


class GnomeInhibitor(BaseInhibitor):
    def __init__(self):
        super().__init__()
        self.bus = dbus.SessionBus()

        self.__proxy = None
        self.__cookie = None

    def inhibit(self, reason=INHIBITION_REASON) -> None:
        if not self.__proxy:
            self.__proxy = self.bus.get_object(
                "org.gnome.SessionManager",
                "/org/gnome/SessionManager",
            )
            self.__proxy = dbus.Interface(
                self.__proxy,
                dbus_interface="org.gnome.SessionManager",
            )

        self.__cookie = self.__proxy.Inhibit(
            "Caffeine",
            dbus.UInt32(0),
            INHIBITION_REASON,
            dbus.UInt32(4),
        )
        self.running = True

    def uninhibit(self) -> None:
        if self.__cookie is not None:
            self.__proxy.Uninhibit(self.__cookie)
        self.running = False

    @property
    def applicable(self) -> bool:
        return "org.gnome.SessionManager" in self.bus.list_names()


class XdgScreenSaverInhibitor(BaseInhibitor):
    def __init__(self):
        super().__init__()
        self.bus = dbus.SessionBus()

        self.__cookie = None

    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.__proxy = self.bus.get_object(
            "org.freedesktop.ScreenSaver",
            "/ScreenSaver",
        )
        self.__proxy = dbus.Interface(
            self.__proxy,
            dbus_interface="org.freedesktop.ScreenSaver",
        )
        self.__cookie = self.__proxy.Inhibit("Caffeine", INHIBITION_REASON)

        self.running = True

    def uninhibit(self) -> None:
        if self.__cookie:
            self.__proxy.UnInhibit(self.__cookie)
        self.running = False

    def is_screen_inhibitor(self):
        return True

    @property
    def applicable(self) -> bool:
        return "org.freedesktop.ScreenSaver" in self.bus.list_names()


class XdgPowerManagmentInhibitor(BaseInhibitor):
    def __init__(self):
        super().__init__()
        self.bus = dbus.SessionBus()

        self.__cookie = None

    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.__proxy = self.bus.get_object(
            "org.freedesktop.PowerManagement",
            "/org/freedesktop/PowerManagement/Inhibit",
        )
        self.__proxy = dbus.Interface(
            self.__proxy,
            dbus_interface="org.freedesktop.PowerManagement.Inhibit",
        )
        self.__cookie = self.__proxy.Inhibit("Caffeine", INHIBITION_REASON)
        self.running = True

    def uninhibit(self) -> None:
        if self.__cookie:
            self.__proxy.UnInhibit(self.__cookie)
        self.running = False

    @property
    def applicable(self) -> bool:
        return "org.freedesktop.PowerManagement" in self.bus.list_names()


class XssInhibitor(BaseInhibitor):
    class XssInhibitorThread(threading.Thread):
        keep_running = True
        daemon = True

        def run(self):
            logging.info("Running XSS inhibitor thread.")
            while self.keep_running:
                os.system("xscreensaver-command -deactivate")
                time.sleep(50)
            logging.info("XSS inhibitor thread finishing.")

    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.running = True
        self.thread = XssInhibitor.XssInhibitorThread()
        self.thread.start()

    def uninhibit(self) -> None:
        self.running = False
        self.thread.keep_running = False

    @property
    def applicable(self) -> bool:
        # TODO!
        return os.system("pgrep xscreensaver") == 0


class DpmsInhibitor(BaseInhibitor):
    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.running = True

        os.system("xset -dpms")

    def uninhibit(self) -> None:
        self.running = False

        # FIXME: Aren't we enabling it if it was never online?
        # Grep `xset q` for "DPMS is Enabled"
        os.system("xset +dpms")

    def is_screen_inhibitor(self):
        return True

    @property
    def applicable(self) -> bool:
        # TODO: Condition is incomplete
        return os.environ.get("WAYLAND_DISPLAY") is None


class XorgInhibitor(BaseInhibitor):
    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.running = True
        os.system("xset s off")

    def uninhibit(self) -> None:
        self.running = False

        # FIXME: Aren't we enabling it if it was never online?
        # Scrensaver.*\n\s+timeout:  600
        os.system("xset s on")

    @property
    def applicable(self) -> bool:
        # TODO: Condition is incomplete
        return os.environ.get("WAYLAND_DISPLAY") is None


class XautolockInhibitor(BaseInhibitor):
    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.running = True
        os.system("xautolock -disable")

    def uninhibit(self) -> None:
        self.running = False
        os.system("xautolock -enable")

    @property
    def applicable(self) -> bool:
        return os.system("pgrep xautolock") == 0


class XfceInhibitor(BaseInhibitor):
    def __init__(self):
        BaseInhibitor.__init__(self)

    def inhibit(self):
        self.running = True

        os.system(
            "xfconf-query -c xfce4-power-manager"
            "-p /xfce4-power-manager/presentation-mode -s true"
        )

    def uninhibit(self):
        self.running = False

        os.system(
            "xfconf-query -c xfce4-power-manager"
            "-p /xfce4-power-manager/presentation-mode -s false"
        )

    @property
    def applicable(self):
        # TODO!
        return True


class XidlehookInhibitor(BaseInhibitor):
    def inhibit(self, reason=INHIBITION_REASON) -> None:
        self.running = True
        os.system("pkill -SIGSTOP xidlehook")

    def uninhibit(self) -> None:
        self.running = False
        os.system("pkill -SIGCONT xidlehook")

    @property
    def applicable(self) -> bool:
        return os.system("pgrep xidlehook") == 0



# Copyright (c) 2014 Hugo Osvaldo Barrera
# Copyright © 2009 The Caffeine Developers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import os


class ProcManager:
    def __init__(self, persistence_file):
        self.persistence_file = persistence_file
        self.proc_list = []

        if os.path.exists(self.persistence_file):
            self.import_proc(self.persistence_file)

    def get_process_list(self):
        return self.proc_list[:]

    def add_proc(self, name):
        if name not in self.proc_list:
            self.proc_list.append(name)
        self.save()

    def remove_proc(self, name):
        self.proc_list.remove(name)
        self.save()

    def import_proc(self, filename):
        for line in open(filename):
            line = line.strip()
            if line not in self.proc_list:
                self.proc_list.append(line)
        self.save()

    def save(self):
        self.proc_list.sort()
        with open(self.persistence_file, "w") as f:
            f.write("\n".join(self.proc_list))



# Copyright (c) 2014 Hugo Osvaldo Barrera
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import os
from os import makedirs
from os.path import exists
from os.path import join

try:
    from xdg.BaseDirectory import xdg_config_home
except ModuleNotFoundError:
    from xdg import xdg_config_home

    xdg_config_home = str(xdg_config_home())

PACKAGE_PATH = os.path.dirname(__file__)
LOCALE_PATH = join(PACKAGE_PATH, "locale")
GLADE_PATH = join(PACKAGE_PATH, "assets/glade")
IMAGE_PATH = join(PACKAGE_PATH, "assets/images")
ICON_PATH = join(PACKAGE_PATH, "assets/icons")


def get_glade_file(filename):
    return join(GLADE_PATH, filename)


def get_config_dir():
    return __config_dir


def get_whitelist_file():
    return join(__config_dir, "whitelist.txt")


def get_blacklist_file_audio():
    return join(__config_dir, "audio_blacklist.txt")


__config_dir = join(xdg_config_home, "caffeine")


if not exists(__config_dir):
    makedirs(__config_dir)

if not exists(get_whitelist_file()):
    open(get_whitelist_file(), "a").close()

if not exists(get_blacklist_file_audio()):
    open(get_blacklist_file_audio(), "a").close()
